/* This file was generated by syncqt. */
#ifndef QT_QTINPUTSUPPORT_VERSION_H
#define QT_QTINPUTSUPPORT_VERSION_H

#define QTINPUTSUPPORT_VERSION_STR "5.15.13"

#define QTINPUTSUPPORT_VERSION 0x050F0D

#endif // QT_QTINPUTSUPPORT_VERSION_H
